import { ILifetime } from '../Lifetime';
import { ServiceCollection } from './ServiceCollection';
import { Factory } from './types';
export declare enum MockStrategy {
    /**
     * ONLY HERE FOR DOCUMENTATION PURPOSES, DO NOT USE IN CODE
     *
     * Default behaviour for any dependency/dependency property given in the ProviderMock that isn't a MockStrategy
     * Getter usage will return most recent dummy value (either the initial value, or a later set value)
     * Setter usage will set new dummy value
     */
    dummyStub = "dummyStub",
    /**
     * This is the default behaviour unless otherwise explicitly stated
     *
     * Getter usage will return null
     * Setter usage will do nothing
     */
    nullStub = "nullStub",
    /**
     * Getter usage will throw NotMockedException(dependency, prop, usage-type)
     * Setter usage will throw NotMockedException(dependency, prop, usage-type)
     */
    exceptionStub = "exceptionStub"
}
declare type PropertyMock<T> = {
    [key in keyof T]?: T[key] | MockStrategy | null | (() => null);
};
declare type DependencyMock<E, K extends keyof E> = Partial<PropertyMock<E[K]>> | Factory<Partial<PropertyMock<E[K]>>, E> | MockStrategy;
export declare type ProviderMock<E> = {
    [key in keyof E]?: DependencyMock<E, key>;
};
export declare function proxyLifetimes<E>(services: ServiceCollection<E>, providerMock: MockStrategy | ProviderMock<E>, defaultMock?: MockStrategy): import("../ServiceProvider").IServiceProvider<E>;
export declare function proxyLifetime<E>(lifetime: ILifetime<unknown, E>, dependencyMock: DependencyMock<E, keyof E>, defaultMock: MockStrategy): ILifetime<unknown, E>;
export declare function mockDependency<T extends object>(name: string, shadowInstance: any, propertyMock: PropertyMock<T>, defaultMock: MockStrategy): T;
export {};
