"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScopedServiceProvider = void 0;
var Errors_1 = require("../Errors");
var utils_1 = require("../utils");
var ScopedServiceProvider = /** @class */ (function () {
    function ScopedServiceProvider(parent, lifetime) {
        Object.defineProperty(this, "instances", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "root", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_isDone", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "lookup", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "ordered", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "proxy", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "parent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_lastSingleton", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.parent = parent;
        this.root = parent.root;
        this.lifetimes = parent.lifetimes;
        this.proxy = (0, utils_1.proxyOf)(this);
        if (parent instanceof ScopedServiceProvider) {
            this.instances = parent.instances;
            this._lastSingleton = parent.lastSingleton;
            this.lookup = __assign({}, parent.lookup);
            this.ordered = parent.ordered.map(function (e) { return e; });
        }
        else {
            this.instances = {};
        }
        if (lifetime)
            this._enter(lifetime);
    }
    Object.defineProperty(ScopedServiceProvider.prototype, "isDone", {
        get: function () {
            return this._isDone;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "depth", {
        get: function () {
            return this.ordered.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "lastSingleton", {
        get: function () {
            return this._lastSingleton;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "using", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (action) {
            action(this.proxy);
            this._isDone = true;
        }
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "provide", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var lifetime = this.getLifetime(selector);
            var context = new ScopedServiceProvider(this, lifetime);
            var result = lifetime.provide(context);
            context._isDone = true;
            return result;
        }
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "enter", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            if (!lifetime)
                return this;
            return new ScopedServiceProvider(this, lifetime);
        }
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "_enter", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            if (lifetime.name in this.lookup) {
                throw new Errors_1.CircularDependencyError(lifetime.name, this.ordered.map(function (e) { return e.name; }));
            }
            this.ordered.push(lifetime);
            this._lastSingleton = lifetime.isSingleton
                ? lifetime
                : this._lastSingleton;
            this.lookup[lifetime.name] = lifetime;
        }
    });
    Object.defineProperty(ScopedServiceProvider.prototype, "getLifetime", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var key = (0, utils_1.extractSelector)(selector);
            var lifetime = this.lifetimes[key];
            if (!lifetime)
                throw new Errors_1.ExistenceDependencyError(key);
            return lifetime;
        }
    });
    return ScopedServiceProvider;
}());
exports.ScopedServiceProvider = ScopedServiceProvider;
